@@ -77,18 +77,16 @@ static int check_file_header(AVIOContext *pb, uint64_t guid)
     return 0;
 }
 
-static void read_string(AVFormatContext *avctx, AVIOContext *pb, const char *tag, unsigned size)
+static void read_string(AVFormatContext *avctx, AVIOContext *pb, const char *tag, int size)
 {
     char * value = av_malloc(size + 1);
-    int ret;
-
     if (!value) {
         avio_skip(pb, size);
         return;
     }
 
-    ret = avio_read(pb, value, size);
-    if (ret != size || !value[0]) {
+    avio_read(pb, value, size);
+    if (!value[0]) {
         av_free(value);
         return;
     }
@@ -344,9 +342,9 @@ static int read_header(AVFormatContext *avctx)
         return ret;
 
     /* scan secondary files */
-    if (strlen(avctx->filename) > 2) {
+    if (strlen(avctx->url) > 2) {
         int i;
-        char *filename = av_strdup(avctx->filename);
+        char *filename = av_strdup(avctx->url);
 
         if (!filename)
             return AVERROR(ENOMEM);
@@ -395,14 +393,10 @@ static int read_packet(AVFormatContext *avctx, AVPacket *pkt)
 {
     MlvContext *mlv = avctx->priv_data;
     AVIOContext *pb;
-    AVStream *st;
+    AVStream *st = avctx->streams[mlv->stream_index];
     int index, ret;
     unsigned int size, space;
 
-    if (!avctx->nb_streams)
-        return AVERROR_EOF;
-
-    st = avctx->streams[mlv->stream_index];
     if (mlv->pts >= st->duration)
         return AVERROR_EOF;
 
@@ -420,25 +414,19 @@ static int read_packet(AVFormatContext *avctx, AVPacket *pkt)
     if (size < 16)
         return AVERROR_INVALIDDATA;
     avio_skip(pb, 12); //timestamp, frameNumber
-    size -= 12;
-    if (st->codecpar->codec_type == AVMEDIA_TYPE_VIDEO) {
-        if (size < 8)
-            return AVERROR_INVALIDDATA;
+    if (st->codecpar->codec_type == AVMEDIA_TYPE_VIDEO)
         avio_skip(pb, 8); // cropPosX, cropPosY, panPosX, panPosY
-        size -= 8;
-    }
     space = avio_rl32(pb);
-    if (size < space + 4LL)
-        return AVERROR_INVALIDDATA;
     avio_skip(pb, space);
-    size -= space;
 
     if ((mlv->class[st->id] & (MLV_CLASS_FLAG_DELTA|MLV_CLASS_FLAG_LZMA))) {
         ret = AVERROR_PATCHWELCOME;
     } else if (st->codecpar->codec_type == AVMEDIA_TYPE_VIDEO) {
         ret = av_get_packet(pb, pkt, (st->codecpar->width * st->codecpar->height * st->codecpar->bits_per_coded_sample + 7) >> 3);
     } else { // AVMEDIA_TYPE_AUDIO
-        ret = av_get_packet(pb, pkt, size - 4);
+        if (space > UINT_MAX - 24 || size < (24 + space))
+            return AVERROR_INVALIDDATA;
+        ret = av_get_packet(pb, pkt, size - (24 + space));
     }
 
     if (ret < 0)
