@@ -126,10 +126,6 @@
 
 #include "libswresample/version.h"
 
-#if LIBSWRESAMPLE_VERSION_MAJOR < 1
-#define SWR_CH_MAX 32   ///< Maximum number of channels
-#endif
-
 /**
  * @name Option constants
  * These constants are used for the @ref avoptions interface for lswr.
